SystemOrganization addCategory: #'Pier-ExternalLogin'!

Notification subclass: #LoginFailedNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ExternalLogin'!

!PRPierConfiguration methodsFor: '*pier-externallogin' stamp: 'kph 8/16/2007 22:45'!
attributeLoginValidationClass
	^ (WAListAttribute key: #loginValidation group: #pier)
		options: PUExternalValidationPlugin withAllSubclasses;
		yourself! !

Object subclass: #PUExternalValidationPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ExternalLogin'!

!PUExternalValidationPlugin commentStamp: 'kph 8/17/2007 01:43' prior: 0!
My subclasses provide the connection between the pier Login (PUELogin) and external validation
resources, e.g. LDAP or a MySql data table of user accounts.

Our model for logging in consists of two databases:

1. a validation 'accountDetails' database, which handles the user/password look up and is
looked after by the IT/site infrastucture. 

2. a domain 'userData' database which holds user information and is 
managed by the application client.

Plugins implement the following API.

MyValidationPlugin-class-loginFor: aLoginCommand

	returns an 'accountDetails' object for the user, or nil if the validation failed.

MyValidationPlugin-class-userFrom: aCommand details: details

	Default behaviour picks a user (named by accountDetails-#pierRole from the pier-internal users to use as a template for the external user.
	The user object itself is only for this session, it does not get permanently added to the internal kernel users table.

MyValidationPlugin-class-userDataFor: user with: accountDetails

	returns a 'user data' domain object for the user for use in the application.
	This 'user data' domain object is given a final chance to refuse login, it can throw an exception
	in its userData-#validateLogin (this allows the domain database to include such things as an account expiry date etc)
	
PULoginWithExternalValidation puts accountDetails and userData objects into properties in the PUUser object for this session.	
	
	

!

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 11/30/2007 22:17'!
forLogin: aLoginCommand in: context

	LoginFailedNotification signal: 'Invalid Login'! !

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 23:14'!
invalidLoginRole: role notFoundForUser: username
	PUInvalidLogin
		description: self description
		signal: 'Role: ', role, ' for user: ', (username ifNil: [ 'not-as-yet-defined' ]), ' not found'
! !

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 23:46'!
loginDescription: desc

	" you can annotate the login description, see Magritte-ComponentDecoration"
	
	^ desc! !

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 11/16/2007 21:51'!
pierUserFrom: aCommand result: details

	| prototypeUserName prototype |
	
	prototypeUserName := details pierRole.
	 
	prototype := aCommand kernel users detect: [ :userPrototype  | userPrototype name = prototypeUserName ] 
								ifNone: [ self invalidLoginRole: details pierRole notFoundForUser: details username ].
					 							 
	^ prototype newFromPrototype
				setName: details username;
				yourself
	! !

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 11/16/2007 21:52'!
setUpPierUser: pierUser result: loginResult 
	 ! !

PULogin subclass: #PUELogin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ExternalLogin'!

!PUELogin commentStamp: 'kph 8/16/2007 23:46' prior: 0!
I validate the login request against the in-image pier users, and then against an external resource. See class comment for PUExternalValidationPlugin

Returning an object which contains the retrieved database record for that user we instanciate a 
temporary pier user from it, using an existing 'role-user' as a prototype.

The roles defined in the db refer to internal user prototypes prefixed with 'role-' in order to prevent 
accidentally (on purpose) spoofing of an in-image only admin user.

This allows the access priviliges to be managed entirely within pier using a manageable number of users, which is handy given that there are many thousand accounts defined in the users validation table.

The chosen plugin also gets the opportunity to annotate my description (see Magritte-ComponentDecoration)!

!PUELogin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 22:25'!
isAbstract

	^ false! !

!PUELogin class methodsFor: 'as yet unclassified' stamp: 'kph 8/9/2007 03:18'!
label
	^ 'Login'! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 1/29/2008 13:26'!
description

	^ (self validationPlugin ifNil: [ ^ super description ]) loginDescription: super description! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 1/30/2008 15:26'!
doExternalValidation

	 | validation loggedinUser loginResult | 
		 
	validation := self validationPlugin ifNil: [ self invalidLogin: 'Invalid Login' ].	
	
	self password ifNil:[ ^ self invalidLogin: 'Please enter password.' ].
	self username ifNil:[ ^ self invalidLogin: 'Please enter user name.' ].
	 
	[ loginResult := validation forLogin: self in: context ] on: LoginFailedNotification do: [ :ex | ^ self invalidLogin: ex messageText ].
			
	loggedinUser := validation pierUserFrom: self result: loginResult.
	
	validation setUpPierUser: loggedinUser result: loginResult.
		
	^ loggedinUser

		
	! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 11/10/2007 05:51'!
doValidate

	super doValidate.
	
	"if we are an admin user show the toolbar helper (if enabled)"							
	(self user groups includes: self kernel adminGroup) ifTrue: [ 
		self context session showToolbarHelper: true.
	].! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 8/16/2007 16:01'!
invalidLogin

	^ self doExternalValidation! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 11/16/2007 21:21'!
invalidLogin: message
	PUInvalidLogin
		description: self description
		signal: message! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 8/16/2007 22:53'!
validationPlugin
	
	^ self context session application preferenceAt: #loginValidation.
	
	
	! !

!PULogin class methodsFor: '*pier-externallogin-override' stamp: 'kph 8/17/2007 00:04'!
isAbstract

	"extemely cheeky"
	^ true! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 9/19/2007 11:58'!
accountDetails

	^ self propertyAt: #accountDetails ifAbsent: [ nil ]! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 9/19/2007 11:59'!
accountDetails: v

	^ self propertyAt: #accountDetails put: v! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 10/4/2007 09:11'!
includesGroup: aGroup

	^ self groups includes: aGroup! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 9/19/2007 12:05'!
like: user

user ifNil: [ ^false ].

^ self prototype = user prototype! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 9/19/2007 11:47'!
newFromPrototype 
	"instanciate a new user using this one as a protoype"
	
	^ (self copy) prototype: self; yourself! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 9/19/2007 11:45'!
prototype 
	"if we have a prototype then return it if not then we are it"
	
	^ self propertyAt: #prototype ifAbsent: self
! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 9/19/2007 11:44'!
prototype: n
	"when a user instance is created from a prototype user instance, the original
	 name is saved here for future reference"
	
	self propertyAt: #prototype put: n
! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 9/19/2007 12:00'!
userData

	^ self propertyAt: #userData ifAbsent: [ nil ]! !

!PUUser methodsFor: '*pier-externallogin' stamp: 'kph 9/19/2007 12:00'!
userData: v

	^ self propertyAt: #userData put: v! !

!PUSecurity methodsFor: '*pier-externallogin-override' stamp: 'kph 6/11/2008 01:41'!
isAllowedCommand: aCommandClass in: aContext
 
	(aContext user notNil and: [ aContext user isSuperuser ])
		ifTrue: [ ^ true ].
	((self owner ifNil: [ ^ true ]) like: aContext user)
		ifTrue: [ ^ self allows: aCommandClass permissions: self ownerPermissions ].
	((self group notNil and: [ aContext user notNil]) and: [ aContext user includesGroup: self group  ])
		ifTrue: [ ^ self allows: aCommandClass permissions: self groupPermissions ].
	^ self allows: aCommandClass permissions: self otherPermissions! !

!PUSecurity methodsFor: '*pier-externallogin-override' stamp: 'kph 1/29/2008 13:22'!
validateCommand: aCommand

	| cUser |
	cUser := aCommand context user.

	(cUser notNil
		and: [ cUser isSuperuser ])
			ifTrue: [ ^ self ].
	(((self owner ifNil: [ ^ cUser isNil ]) like: cUser)
			and: [ self ownerPermissions includes: aCommand permission ])
			ifTrue: [ ^ self ].
				((self group notNil and: [ cUser notNil]) 
						and: [ (cUser includesGroup: self group)
									and: [ self groupPermissions includes: aCommand permission ] ])
											ifTrue: [ ^ self ].
	
	(self otherPermissions includes: aCommand permission)
		ifTrue: [ ^ self ].
	MAConditionError description: self description signal: 'Insufficient permissions'! !
